@@ -18,7 +18,7 @@ use POSIX             qw/strftime/;
 use Scalar::Util      qw/blessed reftype/;
 
 my @default_modifiers   = (
-	qr/\%\S+/       => \&_modif_format,
+	qr/\% ?\S+/     => \&_modif_format,
 	qr/BYTES\b/     => \&_modif_bytes,
 	qr/HTML\b/      => \&_modif_html,
 	qr/YEAR\b/      => \&_modif_year,
@@ -26,6 +26,7 @@ my @default_modifiers   = (
 	qr/DT\b/        => \&_modif_dt,
 	qr/DATE\b/      => \&_modif_date,
 	qr/TIME\b/      => \&_modif_time,
+	qr/\=/          => \&_modif_name,
 	qr!//(?:\"[^"]*\"|\'[^']*\'|\w+)! => \&_modif_undef,
 );
 
@@ -69,7 +70,7 @@ String::Print - printf alternative
   printp 'dump: %s', \%settings;
 
   # modifiers
-  printi 'price: {price%.2f}', price => 3.14*VAT*EURO;
+  printi 'price: {price%.2f}', price => 3.14 * EURO;
 
   # [0.91] more complex interpolation names
   printi 'filename: {c.filename}', c => \%config;
@@ -78,7 +79,7 @@ String::Print - printf alternative
 
   ### Object Oriented interface
 
-  use String::Print 'oo';      # import nothing
+  use String::Print 'oo', %config;      # import no functions
   my $f = String::Print->new(%config);
   $f->printi('age {years}', years => 12);
   $f->printp('age %d', 12);
@@ -106,8 +107,6 @@ module!
 
 =chapter METHODS
 
-=section The Object Oriented interface
-
 See functions M<printi()>, M<sprinti()>, M<printp()>, and M<sprintp()>: you
 can also call them as method.
 
@@ -124,7 +123,7 @@ in multiple source files, or when you need different configurations
 within one program.  In these cases, the hassle of explicitly using the
 object has some benefits.
 
-=subsection Constructors
+=section Constructors
 
 =c_method new %options
 The %options of the constructure configure processing options.
@@ -159,7 +158,7 @@ inserted.  May can overrule that behavior.
     encode_for  => 'HTML',
   );
 
-  $f->printi("price: {p EUR}", p => 3.1415); # price: XX3.14 e (X=blank)
+  $f->printi("price: {p EUR}", p => 3.1415); # price: ␣␣3.14 e
   $f->printi("count: {c}", c => undef);      # count: -
 =cut
 
@@ -208,7 +207,7 @@ sub import(@)
 }
 
 #--------------------
-=subsection Attributes
+=section Attributes
 
 =method addModifiers PAIRS
 The PAIRS are a combination of an selector and a CODE which processes the
@@ -255,7 +254,7 @@ sub encodeFor($)
 # OODoc does not like it when we have methods and functions with the same name.
 
 #--------------------
-=subsection Printing
+=section Printing
 
 The following are provided as method and as function.  You find their
 explanation further down on this page.
@@ -359,9 +358,10 @@ sub sprinti($@)
 
 sub _expand($$$)
 {	my ($self, $key, $modifier, $args) = @_;
+	local $args->{varname} = $key;
 
 	my $value;
-	if(index($key, '.')== -1)
+	if(index($key, '.') == -1)
 	{	# simple value
 		$value = exists $args->{$key} ? $args->{$key} : $self->_missingKey($key, $args);
 		$value = $value->($self, $key, $args)
@@ -439,24 +439,8 @@ sub _reportMissingKey($$)
 }
 
 # See dedicated section in explanation in DETAILS
-sub _modif_format($$$$)
-{	my ($self, $format, $value, $args) = @_;
-	defined $value && length $value or return undef;
-
-	use locale;
-	if(ref $value eq 'ARRAY')
-	{	@$value or return '(none)';
-		return +[ map $self->_format_print($format, $_, $args), @$value ];
-	}
-	elsif(ref $value eq 'HASH')
-	{	keys %$value or return '(none)';
-		return +{ map +($_ => $self->_format_print($format, $value->{$_}, $args)), keys %$value } ;
-	}
-
-	$format =~ m/^\%([-+ ]?)([0-9]*)(?:\.([0-9]*))?([sS])$/
-		or return sprintf $format, $value;   # simple: not a string
-
-	my ($padding, $width, $max, $u) = ($1, $2, $3, $4);
+sub _modif_format_s($$$$$)
+{	my ($value, $padding, $width, $max, $u) = @_;
 
 	# String formats like %10s or %-3.5s count characters, not width.
 	# String formats like %10S or %-3.5S are subject to column width.
@@ -488,6 +472,45 @@ sub _modif_format($$$$)
 	:                   (' ' x $pad) . $s->as_string;
 }
 
+sub _modif_format_d($$$$)
+{	my ($value, $padding, $max, $sep) = @_;
+	my $d = sprintf "%d", $value;   # what perl usually does with floats etc
+	my $v = reverse(reverse($d) =~ s/([0-9][0-9][0-9])/$1$sep/gr);
+	$v =~ s/^\.//;
+
+	if($d !~ /^\-/)
+	{	$v = "+$v" if $padding eq '+';
+		$v = " $v" if $padding eq ' ';
+	}
+	$max or return $v;
+
+	my $pad = $max - length $v;
+
+	    $pad <= 0       ? $v 
+	  : $padding eq '-' ? $v . (' ' x $pad)
+	  : $padding eq ''  ? (' ' x $pad) . $v
+	  :   $v;
+}
+
+sub _modif_format($$$$)
+{	my ($self, $format, $value, $args) = @_;
+	defined $value && length $value or return undef;
+
+	use locale;
+	if(ref $value eq 'ARRAY')
+	{	@$value or return '(none)';
+		return +[ map $self->_format_print($format, $_, $args), @$value ];
+	}
+	elsif(ref $value eq 'HASH')
+	{	keys %$value or return '(none)';
+		return +{ map +($_ => $self->_format_print($format, $value->{$_}, $args)), keys %$value } ;
+	}
+
+	  $format =~ m/^\%(\-?)([0-9]*)(?:\.([0-9]*))?([sS])$/ ? _modif_format_s($value, $1, $2, $3, $4)
+	: $format =~ m/^\%([+\ \-]?)([0-9]*)([_,.])d$/ ? _modif_format_d($value, $1, $2, $3)
+	:    return sprintf $format, $value;   # simple: standard perl sprintf()
+}
+
 # See dedicated section in explanation in DETAILS
 sub _modif_bytes($$$)
 {	my ($self, $format, $value, $args) = @_;
@@ -516,11 +539,12 @@ sub _modif_html($$$)
 
 # Be warned: %F and %T (from C99) are not supported on Windows
 my %dt_format = (
-	ASC     => '%a %b %e %H:%M:%S %Y',
-	ISO     => '%Y-%m-%dT%H:%M:%S%z',
-	RFC2822 => '%a, %d %b %Y %H:%M:%S %z',
-	RFC822  => '%a, %d %b %y %H:%M:%S %z',
-	FT      => '%Y-%m-%d %H:%M:%S',
+	ASC     => '%a %b %e %T %Y',
+	ISO     => '%Y-%m-%dT%T%z',
+	RFC822  => '%a, %d %b %y %T %z',
+	RFC2822 => '%a, %d %b %Y %T %z',
+	RFC5322 => '%a, %d %b %Y %T %z',
+	FT      => '%Y-%m-%d %T',
 );
 
 sub _modif_year($$$)
@@ -584,11 +608,16 @@ sub _modif_undef($$$)
 	$format =~ m!//"([^"]*)"|//'([^']*)'|//(\w*)! ? $+ : undef;
 }
 
+sub _modif_name($$$)
+{	my ($self, $format, $value, $args) = @_;
+	"$args->{varname}$format$value";
+}
+
 =function printi [$fh], $format, %data|\%data
 Calls M<sprinti()> to fill the %data into $format, and
 then sends it to the $fh (by default the selected file)
 
-  open my $fh, '>', $file;
+  open my $fh, '>:encoding(UTF-8)', $file;
   printi $fh, ...
 
   printi \*STDERR, ...
@@ -650,13 +679,15 @@ sub _printp_rewrite($)
 			next;
 		}
 
-		$printp =~ s/\%(?:([0-9]+)\$)?      # 1=positional
-						([-+0 \#]*)         # 2=flags
-						([0-9]*|\*)?        # 3=width
-						(?:\.([0-9]*|\*))?  # 4=precission
-						(?:\{ ([^}]*) \})?  # 5=modifiers
-						(\w)                # 6=conversion
-					//x
+		$printp =~ s/
+			\%
+			(?:([0-9]+)\$)?     # 1=positional
+			([-+0 \#]*)         # 2=flags
+			([0-9]*|\*)?        # 3=width
+			(?:\.([0-9]*|\*))?  # 4=precission
+			(?:\{ ([^}]*) \})?  # 5=modifiers
+			(\w)                # 6=conversion
+		//x
 			or die "format error at '$printp' in '$params[0]'";
 
 		$pos       = $1 if $1;
@@ -684,7 +715,10 @@ sub sprintp(@)
 #--------------------
 =chapter DETAILS
 
-=section Why use C<printi()>, not C<printf()>?
+  Your manual-page reader may not support the unicode used
+  in some of the examples below.
+
+=section Why use C<printi()> to replace C<printf()>?
 
 The C<printf()> function is provided by Perl's CORE; you do not need
 to install any module to use it.  Why would you use consider using
@@ -748,8 +782,6 @@ Simplified:
 
 Example:
 
-  #XXX Your manual-page reader may not support the unicode used
-  #XXX in the examples below.
   printi "price: {price € %-10s}", price => $cost;
   printi "price: {price € %-10s}", { price => $cost };
   printp "price: %-10{€}s", $cost;
@@ -769,7 +801,6 @@ separated by dots (no blanks!)
 B<Please> use explanatory key names, to help the translation
 process once you need that (in the future).
 
-
 =subsection Simple keys
 
 A simple key directly refers to a named parameter of the function or method:
@@ -785,7 +816,6 @@ You may also pass them as HASH or CODE:
 
 The smartness of pre-processing CODE is part of serialization.
 
-
 =subsection Complex keys
 
 [0.91] In the previous section, we kept our addressing it simple: let's
@@ -898,10 +928,12 @@ objects get stringified.
 
 =section Interpolation: Modifiers
 
-Modifiers are used to change the value to be inserted, before the characters
-get interpolated in the line.  This is a powerful simplification.  Let's
-discuss this with an example.
+Modifiers are used to change the value to be inserted, before
+the characters get interpolated in the line.  This is a powerful
+simplification.  Some useful modifiers are already provided by default.
+They are also good examples how to write your own.
 
+Let's discuss this with an example.
 In traditional (gnu) gettext, you would write:
 
   printf(gettext("approx pi: %.6f\n"), PI);
@@ -943,10 +975,7 @@ Above example in M<printp()> syntax, shorter but less maintainable:
   printp "%s %2d %-8s 10d %s\n",
      '-rw-r--r--', 7, 'me', 12345, $filename;
 
-
-=section Interpolation: default modifiers
-
-=subsection Default modifier: POSIX format
+=subsection Modifier: POSIX format starts with '%'
 
 As shown in the examples above, you can specify a format.  This can,
 for instance, help you with rounding or columns:
@@ -955,7 +984,7 @@ for instance, help you with rounding or columns:
   printp "weight is {kilogram%d}", kilogram => 127*OUNCE_PER_KILO;
   printp "{filename%-20.20s}\n", filename => $fn;
 
-=subsubsection - improvements on POSIX format
+=subsubsection POSIX modifier extension '%S'
 
 The POSIX C<printf()> does not handle unicode strings.  Perl does
 understand that the 's' modifier may need to insert utf8 so does not
@@ -972,8 +1001,24 @@ specify its width, then there is no performance penalty for using 'S'.
   # name right aligned, commas on same position, always
   printp "name: {name%20S},\n", name => $some_chinese;
 
+=subsubsection POSIX modifier extensions '%[+ -]?[0-9]*[_,.]d'
 
-=subsection Default modifier: BYTES
+[0.96] Only available when you print (big) decimals: add an underscore,
+comma, or dot on the thousands.
+
+  printi "{count%_d}\n", count => 1e9;   # 1_000_000_000
+  printi "{count%,d}\n", count => 1e9;   # 1,000,000,000
+  printi "{count%.d}\n", count => 1e9;   # 1.000.000.000
+
+  printi "'{v%10.d}'",  v =>  10000;   # '    10.000';
+  printi "'{v%10_d}'",  v => -10000;   # '   -10_000';
+  printi "'{v%-10.d}'", v =>  10000;   # '10.000    ';
+  printi "'{v%-10.d}'", v => -10000;   # '-10.000   ';
+  printi "'{v%+10,d}'", v =>  10000;   # '   +10,000';
+  printi "'{v% ,d}'",   v =>  10000;   # ' 10,000';
+  printi "'{v% ,d}'",   v => -10000;   # '-10,000';
+
+=subsection Modifier: BYTES
 
 [0.91] Too often, you have to translate a (file) size into humanly readible
 format.  The C<BYTES> modifier simplifies this a lot:
@@ -983,11 +1028,11 @@ format.  The C<BYTES> modifier simplifies this a lot:
 The output will always be 6 characters.  Examples are "999  B", "1.2 kB",
 and " 27 MB".
 
-=subsection Default modifier: HTML
+=subsection Modifier: HTML
 
 [0.95] interpolate the parameter with HTML entity encoding.
 
-=subsection Default modifiers: YEAR, DATE, TIME, DT, and DT()
+=subsection Modifiers: YEAR, DATE, TIME, DT, and DT()
 
 [0.91] A set of modifiers help displaying dates and times.  They are a
 little flexible in values they accept, but do not expect miracles: when
@@ -1038,14 +1083,15 @@ The short form C<DT> is an alias for C<DT(FT)>.  The DT modifier can
 produce different formats:
 
   DT(ASC)     : %a %b %e %T %Y       asctime output
-  DT(FT)      : %F %T                YYYY-MM-DD HH:mm:ss
+  DT(FT)      : %F %T                YYYY-MM-DD HH:MM:SS
   DT(ISO)     : %FT%T%z              iso8601
   DT(RFC822)  : %a, %d %b %y %T %z   email old
   DT(RFC2822) : %a, %d %b %Y %T %z   email newer
+  DT(RFC5322) : %a, %d %b %Y %T %z   email newest [0.96]
 
 You may suggest additional formats, or add your own modifier.
 
-=subsection Default modifiers: //word, //"string", //'string'
+=subsection Modifier: //word, //"string", //'string'
 
 [0.91] By default, an undefined value is shown as text 'undef'.  Empty
 strings are shown as nothing.  This may not be nice.  You may want to
@@ -1062,6 +1108,14 @@ different kinds of output:
   "price: {price//5 EUR}"
   "price: {price EUR//unknown}"
 
+=subsection Modifier: '='
+
+[0.96] As (always trailing) modifier, this will show the interpolated
+name before the value.  It might simplify debugging statements.
+
+  printi "visitors: {count=}", count => 1;      # visitors: count=1
+  printi "v: {count %-8,d =}X", count => 10000; # v: count=10,000␣␣X
+
 =subsection Private modifiers
 
 You may pass your own modifiers.  A modifier consists of a selector and
@@ -1107,7 +1161,7 @@ Another example.  Now, we want to add timestamps.  In this case, we
 decide for modifier names in C<\w>, so we need a blank to separate
 the parameter from the modifer.
 
-=subsection Modifiers: stacking
+=subsection Stacking modifiers
 
 You can add more than one modifier.  The modifiers detect the extend of
 their own information (via a regular expression), and therefore the
@@ -1116,10 +1170,10 @@ formatter understands where one ends and the next begins.
 The modifiers are called in order:
 
   printi "price: {p€%9s}\n", p => $p; # price: ␣␣␣123.45
-  printi ">{t T%10s}<", t => $now;    # >␣␣12:59:17<
+  printi "!{t T%10s}!", t => $now;    # !␣␣12:59:17!
 
   printp "price: %9{€}s\n", $p;       # price: ␣␣␣123.45
-  printp ">%10{T}s<", $now;           # >␣␣12:59:17<
+  printp "!%10{T}s!", $now;           # !␣␣12:59:17!
 
 
 =section Output encoding
@@ -1129,7 +1183,7 @@ to insert (translated) strings with parameters into HTML templates.
 You can imagine that some of the parameter may need to be encoded to
 HTML in the template, and other not.
 
-=subsection example with Log::Report::Template
+=subsection Example with Log::Report::Template
 
 In pure Template Toolkit, you would write
 
